@@ -63,3 +63,17 @@ CUTLASS_INCLUDE_DIRS = [
     _package_root / "data" / "cutlass" / "tools" / "util" / "include",
 ]
 SPDLOG_INCLUDE_DIR = _package_root / "data" / "spdlog" / "include"
+
+
+def get_nvshmem_include_dir():
+    import nvidia.nvshmem
+
+    path = pathlib.Path(nvidia.nvshmem.__path__[0]) / "include"
+    return path
+
+
+def get_nvshmem_lib_dir():
+    import nvidia.nvshmem
+
+    path = pathlib.Path(nvidia.nvshmem.__path__[0]) / "lib"
+    return path
