@@ -14,7 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from functools import cache
+import functools
 from typing import Any, Optional, Tuple
 
 import torch
@@ -24,8 +24,6 @@ from .jit import env as jit_env
 from .jit import gen_jit_spec
 from .utils import register_custom_op, register_fake_op
 
-_rope_module = None
-
 
 def gen_rope_module() -> JitSpec:
     return gen_jit_spec(
@@ -37,19 +35,9 @@ def gen_rope_module() -> JitSpec:
     )
 
 
+@functools.cache
 def get_rope_module():
-    global _rope_module
-    if _rope_module is None:
-        _rope_module = gen_rope_module().build_and_load()
-    return _rope_module
-
-
-@cache
-def get_module_attr(attr: str) -> Any:
-    global _rope_module
-    if _rope_module is None:
-        get_rope_module()
-    return getattr(_rope_module, attr).default
+    return gen_rope_module().build_and_load()
 
 
 @register_custom_op("flashinfer::apply_rope", mutates_args=("q_rope", "k_rope"))
@@ -65,9 +53,7 @@ def _apply_rope(
     rope_scale: float,
     rope_theta: float,
 ) -> None:
-    indptr = indptr.int()
-    offsets = offsets.int()
-    get_module_attr("apply_rope")(
+    get_rope_module().apply_rope(
         q,
         k,
         q_rope,
@@ -113,9 +99,7 @@ def _apply_llama31_rope(
     high_freq_factor: float,
     old_context_len: float,
 ) -> None:
-    indptr = indptr.int()
-    offsets = offsets.int()
-    get_module_attr("apply_llama31_rope")(
+    get_rope_module().apply_llama31_rope(
         q,
         k,
         q_rope,
@@ -163,8 +147,7 @@ def _apply_rope_pos_ids(
     rope_scale: float,
     rope_theta: float,
 ) -> None:
-    pos_ids = pos_ids.int()
-    get_module_attr("apply_rope_pos_ids")(
+    get_rope_module().apply_rope_pos_ids(
         q,
         k,
         q_rope,
@@ -204,8 +187,7 @@ def _apply_rope_pos_ids_cos_sin_cache(
     pos_ids: torch.Tensor,
     interleave: bool,
 ) -> None:
-    pos_ids = pos_ids.int()
-    get_module_attr("apply_rope_pos_ids_cos_sin_cache")(
+    get_rope_module().apply_rope_pos_ids_cos_sin_cache(
         q,
         k,
         q_rope,
@@ -247,8 +229,7 @@ def _apply_llama31_rope_pos_ids(
     high_freq_factor: float,
     old_context_len: float,
 ) -> None:
-    pos_ids = pos_ids.int()
-    get_module_attr("apply_llama31_rope_pos_ids")(
+    get_rope_module().apply_llama31_rope_pos_ids(
         q,
         k,
         q_rope,
